@@ -44,6 +44,7 @@ define('CGGM_NEWJS_TEMPLATE','cggm_newjs_template');
 define('CGGM_NEWSIDEBAR_TEMPLATE','cggm_newsidebar_template');
 define('CGGM_NEWCATEGORY_TEMPLATE','cggm_newcategory_template');
 define('CGGM_NEWDIRFORM_TEMPLATE','cggm_newdirform_template');
+define('CGGM_NEWBALOON_TEMPLATE','cggm_baloon_template');
 
 /**
  * Add a category to the map.
@@ -124,7 +125,7 @@ function cggm_add_dynpoint($params, &$smarty)
       return; 
     }
   $name = $params['name'];
-  
+
   $description = '';
   if( isset($params['desc']) )
     {
@@ -217,7 +218,7 @@ class CGGoogleMaps extends CGExtensions
    ---------------------------------------------------------*/
   function GetVersion()
   {
-    return '2.0.3';
+    return '2.1';
   }
 
 
@@ -326,7 +327,7 @@ EOT;
    ---------------------------------------------------------*/
   function GetDependencies()
   {
-    return array('CGExtensions'=>'1.21.5',
+    return array('CGExtensions'=>'1.22.2',
 		 'CGSimpleSmarty'=>'1.4');
   }
 
@@ -336,7 +337,7 @@ EOT;
    ---------------------------------------------------------*/
   function MinimumCMSVersion()
   {
-    return "1.8.2";
+    return "1.9.1";
   }
 	
 	
@@ -407,19 +408,15 @@ EOT;
    ---------------------------------------------------------*/
   function GetCoordsFromAddress($address)
   {
-    require_once('lib/GoogleMapAPI-2.4/GoogleMapAPI.class.php');
-
-    global $gCms;
-    $map = '';
-    if( !isset($gCms->variables['cggm_geocode_map']) )
+    $tmp = cggm_address_lookup::lookup($address);
+    if( is_array($tmp) )
       {
-	$gCms->variables['cggm_geocode_map'] = new GoogleMapAPI('cggm_geocode_map',$this->GetName());
+	$data = array();
+	$data['lat'] = $tmp[0];
+	$data['lon'] = $tmp[1];
+	return $data;
       }
-
-    $map =& $gCms->variables['cggm_geocode_map'];
-
-    $res = $map->geoGetCoords($address);
-    return $res;
+    return $tmp;
   }
 
 
