@@ -105,6 +105,18 @@ else if( isset($params['reset_dirform_template']) )
     $this->SetCurrentTab('dirform_template');
     $map->set_dirform_template($this->GetTemplate(CGGM_NEWDIRFORM_TEMPLATE));
   }
+else if( isset($params['reset_baloon_template']) )
+  {
+    cggm_map_operations::update_from_formdata($map,$params);
+    $map->set_map_template($params['map_template']);
+    $map->set_js_template($params['js_template']);
+    $map->set_sidebar_template($params['sidebar_template']);
+    $map->set_category_template($params['category_template']);
+    $map->set_baloon_template($params['baloon_template']);
+
+    $this->SetCurrentTab('baloon_template');
+    $map->set_baloon_template($this->GetTemplate(CGGM_NEWBALOON_TEMPLATE));
+  }
 else if( isset($params['submit']) || isset($params['apply']) )
   {
     cggm_map_operations::update_from_formdata($map,$params);
@@ -113,6 +125,7 @@ else if( isset($params['submit']) || isset($params['apply']) )
     $map->set_sidebar_template($params['sidebar_template']);
     $map->set_category_template($params['category_template']);
     $map->set_dirform_template($params['dirform_template']);
+    $map->set_baloon_template($params['baloon_template']);
 
     // make sure that height, width, and name are specified
     if( $map->get_name() == '' || $map->height <= 0 || $map->width <= 0 )
@@ -125,8 +138,11 @@ else if( isset($params['submit']) || isset($params['apply']) )
 	// todo, check for errors.
       }
 
-    // redirect
-    $this->RedirectToTab($id);
+    if( isset($params['submit']) )
+      {
+	// redirect
+	$this->RedirectToTab($id);
+      }
   }
 
 
