@@ -88,7 +88,7 @@ $output = $generator->generate();
 $errors = $generator->get_errors();
 if( is_array($errors) )
   {
-    echo $this->DisplayErrorMessge(implode('<br/>',$errors));
+    echo $this->DisplayErrorMessage(implode('<br/>',$errors));
   }
 echo $output;
 
