@@ -37,6 +37,15 @@ echo $this->GetDefaultTemplateForm($this,$id,$returnid,
 
 echo '<div style="text-align: center; border-top: 1px dashed black; margin-bottom: 1em; width: 70%;"></div>';
 
+echo cge_template_admin::get_start_template_form($this,$id,$returnid,
+						 CGGM_NEWBALOON_TEMPLATE,
+						 'defaultadmin','default_templates',
+						 $this->Lang('info_sysdflt_baloon_template'),
+						 'orig_baloon_template.tpl',
+						 $this->Lang('info_sysdflt_template'));
+
+echo '<div style="text-align: center; border-top: 1px dashed black; margin-bottom: 1em; width: 70%;"></div>';
+
 echo cge_template_admin::get_start_template_form($this,$id,$returnid,
 						 CGGM_NEWJS_TEMPLATE,
 						 'defaultadmin','default_templates',
