@@ -13,6 +13,9 @@ $lang['apply'] = 'Apply';
 $lang['ask_delete_icon'] = 'Are you sure you want to remove this icon from the list (no files will be deleted)';
 $lang['ask_reset_icons'] = 'Are you sure you want to remove all icons, and reset to factory defaults?';
 
+# B
+$lang['baloon_maptemplate'] = 'Baloon Template';
+
 # C
 $lang['cancel'] = 'Cancel';
 $lang['category_template'] = 'Category View Template';
@@ -199,6 +202,8 @@ $lang['info_scalecontrols'] = 'A scale can optionally be displayed in the lower
 $lang['info_sensor'] = 'Some browsers (particularly mobile browsers) are capable of providing latitude and longitude information.  Do you want to enable this?';
 $lang['info_sidebar'] = 'Should a sidebar containing all map points be displayed?';
 $lang['info_sv_controls'] = 'Should StreetView Controls be displayed on the map?';
+$lang['info_sv_radius'] = 'When Streetview is enabled for a marker, how far (in meters) should it search for a valid streetview location.  This is important in rural areas where StreetView has not necessarily gone through every street.';
+$lang['info_sysdflt_baloon_template'] = 'System Default &quot;Baloon&quot; Template';
 $lang['info_sysdflt_category_template'] = 'System Default &quot;Category List&quot; Template';
 $lang['info_sysdflt_dirform_template'] = 'System Default &quot;Directions Form&quot; Template';
 $lang['info_sysdflt_js_template'] = 'System Default Javascript Template';
@@ -291,6 +296,7 @@ $lang['sidebar_template'] = 'Sidebar Template';
 $lang['small'] = 'Small';
 $lang['submit'] = 'Submit';
 $lang['sv_controls'] = 'Show Streetview Controls';
+$lang['sv_radius'] = 'Streetview Search Radius';
 
 # T
 $lang['tab_advanced'] = 'Advanced';
