@@ -51,6 +51,10 @@ CREATE TABLE GEOCODES (
 
 class GoogleMapAPI {
 
+  const dflt_icon_name = '**default**';
+  const combine_to_tabs = '**tabs**';
+  const combine_flat = '**flat**';
+
     /**
      * PEAR::DB DSN for geocode caching. example:
      * $dsn = 'mysql://user:pass@localhost/dbname';
@@ -238,6 +242,14 @@ class GoogleMapAPI {
      */
     var $window_trigger = 'click';    
 
+    private $_combine_points = 0;
+    private $_point_combine_fudge = 0;
+    private $_combined_points_icon = self::dflt_icon_name;
+    private $_combine_points_method = self::combine_to_tabs;
+    private $_sidebar_list;
+    private $_marker_list;
+    private $_marker_aliases;
+
     /**
      * what server geocode lookups come from
      *
@@ -250,7 +262,7 @@ class GoogleMapAPI {
      * @var string service name
      */
     var $lookup_service = 'GOOGLE';
-	var $lookup_server = array('GOOGLE' => 'maps.google.com', 'YAHOO' => 'api.local.yahoo.com');
+    var $lookup_server = array('GOOGLE' => 'maps.google.com', 'YAHOO' => 'api.local.yahoo.com');
     
     var $driving_dir_text = array(
             'dir_to' => 'Start address: (include addr, city st/region)',
@@ -366,6 +378,7 @@ class GoogleMapAPI {
 	  {
 	    $this->map_var_name = $var_name;
 	  }
+	$this->_marker_aliases = array();
     }
    
     /**
@@ -653,7 +666,21 @@ class GoogleMapAPI {
         $this->overview_control = false;
      }    
     
-    
+
+    /**
+     *
+     * set combine points flag
+     */
+    function set_combine_points($flag = false,$fudge = 0,$icon = '',$method = self::combine_flat)
+    {
+      $this->_combine_points = $flag;
+      $this->_point_combine_fudge = (float)$fudge;
+      if( $icon == '' ) $icon = self::dflt_icon_name;
+      $this->_combined_points_icon = $icon;
+      $this->_combine_points_method = $method;
+    }
+
+
     /**
      * set the lookup service to use for geocode lookups
      * default is YAHOO, you can also use GOOGLE.
@@ -672,8 +699,55 @@ class GoogleMapAPI {
                 break;
         }       
     }
-    
-        
+
+
+    protected function calc_rectangle()
+    {
+      $min_lon = 1000000;
+      $max_lon = -1000000;
+      $min_lat = 1000000;
+      $max_lat = -1000000;
+      if( is_array($this->_markers) && count($this->_markers) )
+	{
+	  foreach( $this->_markers as $one )
+	    {
+	      $min_lon = (float) min($min_lon,$one->get_longitude());
+	      $max_lon = (float) max($max_lon,$one->get_longitude());
+	      $min_lat = (float) min($min_lat,$one->get_latitude());
+	      $max_lat = (float) max($max_lat,$one->get_latitude());
+	    }
+	}
+
+
+      $this->_min_lat = $min_lat;
+      $this->_max_lat = $max_lat;
+      $this->_min_lon = $min_lon;
+      $this->_max_lon = $max_lon;
+      $this->center_lon = (float) ($min_lon + $max_lon ) / 2.0;
+      $this->center_lat = (float) ($min_lat + $max_lat ) / 2.0;
+    }
+
+    public function add_icon(cggm_icon& $icon)
+    {
+      $this->_icons[$icon->get_name()] = $icon;
+    }
+
+
+    public function add_marker(cggm_marker& $marker)
+    {
+      $addr = $marker->get_address();
+      if( $addr != '' )
+	{
+	  $_geocode = $this->getGeocode($addr);
+	  if( $_geocode !== false )
+	    {
+	      $marker->set_coords($_geocode['lat'],$_geocode['lon']);
+	    }
+	}
+      $this->_markers[] = $marker;
+    }
+
+/*
     /**
      * adds a map marker by address
      * 
@@ -681,12 +755,22 @@ class GoogleMapAPI {
      * @param string $title the title display in the sidebar
      * @param string $html the HTML block to display in the info bubble (if empty, title is used)
      */
-    function addMarkerByAddress($address,$title = '',$html = '',$tooltip = '') {
+    function addMarkerByAddress($name,$address,$title = '',$html = '',$tooltip = '',$icon = self::dflt_icon_name,
+				$sidebar_class = '',$sidebar_html = '') {
         if(($_geocode = $this->getGeocode($address)) === false)
 	  {
             return FALSE;
 	  }
-        return $this->addMarkerByCoords($_geocode['lon'],$_geocode['lat'],$title,$html,$tooltip);
+        return $this->addMarkerByCoords($name,$_geocode['lon'],$_geocode['lat'],$title,
+					$html,$tooltip,$icon,$sidebar_class,$sidebar_html);
+    }
+
+    private function clean_for_javascript($input)
+    {
+      $input = addslashes($input);
+      $input = str_replace('\"','"',$input);
+      $input = str_replace(array("\r","\n"),'',$input);
+      return $input;
     }
 
     /**
@@ -700,13 +784,127 @@ class GoogleMapAPI {
      *     array: The title => content pairs for a tabbed info bubble     
      */
     // TODO make it so you can specify which tab you want the directions to appear in (add another arg)
-    function addMarkerByCoords($lon,$lat,$title = '',$html = '',$tooltip = '') {
+    function addMarkerByCoords($name,$lon,$lat,$title = '',$html = '',$tooltip = '',
+			       $icon = self::dflt_icon_name,
+			       $sidebar_class = '',$sidebar_html = '') 
+    {
+      // clean up cruft.
+      //$title = cms_htmlentities($title,'','UTF-8',true);
+      $html = (is_array($html) || strlen($html) > 0) ? $html : $title;
+
+      $tooltip = $this->clean_for_javascript($tooltip);
+      $title = $this->clean_for_javascript($title);
+      $sidebar_html = $this->clean_for_javascript($sidebar_html);
+      if( is_array($html) )
+	{
+	  for( $i = 0; $i < count($html); $i++ )
+	    {
+	      $html[$i] = $this->clean_for_javascript($html[$i]);
+	      $html[$i] = '<div class="cggm_point">'.$html[$i].'</div>';
+	      $html = implode('',$html);
+	    }
+	}
+      else
+	{
+	  $html = '<div class="cggm_point">'.$this->clean_for_javascript($html).'</div>';
+	}
+
+      // come up with a simple, alias type name.
+      $name = munge_string_to_url($name);
+      $name = str_replace('-','_',$name);
+      for( $i = 0; $i < 10; $i++ )
+	{
+	  $new_name = str_replace('__','_',$name);
+	  if( $new_name == $name ) break;
+	  $name = $new_name;
+	}
+      if( isset($this->_markers[$name]) )
+	{
+	  // it already exists.
+	  $suffix = 2;
+	  while( $suffix < 10 && isset($this->_markers[$name.'_'.$suffix]) )
+	    {
+	      $suffix++;
+	    }
+	  $name = $name.'_'.$suffix;
+	}
+
+      if( $this->sidebar )
+	{
+	  if( !is_array($this->_sidebar_list) )
+	    {
+	      $this->_sidebar_list = array();
+	      $this->_marker_list = array();
+	    }
+	  $str = '';
+	  if( $sidebar_class )
+	    {
+	      $str = ' class="'.trim($sidebar_class).'"';
+	    }
+	  if( !$sidebar_html )
+	    {
+	      $sidebar_html = $html;
+	    }
+	  $this->_sidebar_list[$name] = '<li'.$str.'><span class="sidebar_title"><a href="javascript:click_sidebar(\\\''.$name.'\\\')">'.$title.'</a></span><br/>'.$sidebar_html.'</li>';
+	  $this->_marker_list[$name] = $html;
+	}
+
+      if( $this->_combine_points && $this->_point_combine_fudge > 0.0 )
+	{
+	  foreach( $this->_markers as $key => &$tm )
+	    {
+	      // calculate bounding box.
+	      $box_toplat = $tm['lat'] - $this->_point_combine_fudge;
+	      $box_toplon = $tm['lon'] - $this->_point_combine_fudge;
+	      $box_botlat = $tm['lat'] + $this->_point_combine_fudge;
+	      $box_botlon = $tm['lon'] + $this->_point_combine_fudge;
+	      if( $lat >= $box_toplat && $lat <= $box_botlat &&
+		  $lon >= $box_toplon && $lon <= $box_botlon )
+		{
+		  // overlaps an existing point.
+		  switch( $this->_combine_points_method )
+		    {
+		    case self::combine_to_tabs:
+		      if( !is_array($tm['html']) )
+			{
+			  // make single structure into an array.
+			  $tmp = $tm['html'];
+			  $tm['html'] = array();
+			  $tm['html'][$tm['title']] = $tmp;
+			}
+		      
+		      $tm['html'][$title] = $html;
+		      $tm['icon'] = $this->_combined_points_icon;
+		      $tm['npoints']++;
+		      break;
+
+		    case self::combine_flat:
+		      $tm['icon'] = $this->_combined_points_icon;
+		      $tm['npoints']++;
+		      $tm['html'] .= '<div class="cggm_point_separator"></div>'.$html;
+		      break;
+		    }
+
+		  // can only overlap once.
+		  $this->_marker_aliases[$name] = $tm['name'];
+		  return;
+		}
+	    }
+	}
+
+        if( $icon == '' ) $icon = self::dflt_icon_name;
+
+	$_marker['name'] = $name;
         $_marker['lon'] = $lon;
         $_marker['lat'] = $lat;
-        $_marker['html'] = (is_array($html) || strlen($html) > 0) ? $html : $title;
+        $_marker['html'] = $html;
         $_marker['title'] = $title;
         $_marker['tooltip'] = $tooltip;
-        $this->_markers[] = $_marker;
+	$_marker['icon'] = $icon;
+	$_marker['npoints'] = 1;
+
+        $this->_markers[$name] = $_marker;
+	$this->_marker_aliases[$name] = $name;
         $this->adjustCenterCoords($_marker['lon'],$_marker['lat']);
         // return index of marker
         return count($this->_markers) - 1;
@@ -729,6 +927,7 @@ class GoogleMapAPI {
             return false;
         return $this->addPolyLineByCoords($_geocode1['lon'],$_geocode1['lat'],$_geocode2['lon'],$_geocode2['lat'],$color,$weight,$opacity);
     }
+*/
 
     /**
      * adds a map polyline by map coordinates
@@ -787,6 +986,7 @@ class GoogleMapAPI {
         $this->center_lon = (float) $lon;
     }    
 
+/*
     /**
      * generate an array of params for a new marker icon image
      * iconShadowImage is optional
@@ -800,7 +1000,7 @@ class GoogleMapAPI {
      * @param string $infoWindowAnchorX X coordinate for info window anchor point
      * @param string $infoWindowAnchorY Y coordinate for info window anchor point
      */
-    function createMarkerIcon($iconImage,$iconShadowImage = '',$iconAnchorX = 'x',$iconAnchorY = 'x',$infoWindowAnchorX = 'x',$infoWindowAnchorY = 'x') {
+    private function createMarkerIcon($iconName,$iconImage,$iconShadowImage = '',$iconAnchorX = 'x',$iconAnchorY = 'x',$infoWindowAnchorX = 'x',$infoWindowAnchorY = 'x') {
         $_icon_image_path = strpos($iconImage,'http') === 0 ? $iconImage : $_SERVER['DOCUMENT_ROOT'] . $iconImage;
         if(!($_image_info = @getimagesize($_icon_image_path))) {
             die('GoogleMapAPI:createMarkerIcon: Error reading image: ' . $iconImage);   
@@ -826,14 +1026,15 @@ class GoogleMapAPI {
         }
                         
         $icon_info = array(
-                'image' => $iconImage,
-                'iconWidth' => $_image_info[0],
-                'iconHeight' => $_image_info[1],
-                'iconAnchorX' => $iconAnchorX,
-                'iconAnchorY' => $iconAnchorY,
-                'infoWindowAnchorX' => $infoWindowAnchorX,
-                'infoWindowAnchorY' => $infoWindowAnchorY
-                );
+			   'name'=>$iconName,
+			   'image' => $iconImage,
+			   'iconWidth' => $_image_info[0],
+			   'iconHeight' => $_image_info[1],
+			   'iconAnchorX' => $iconAnchorX,
+			   'iconAnchorY' => $iconAnchorY,
+			   'infoWindowAnchorX' => $infoWindowAnchorX,
+			   'infoWindowAnchorY' => $infoWindowAnchorY
+			   );
         if($iconShadowImage) {
             $icon_info = array_merge($icon_info, array('shadow' => $iconShadowImage,
                                                        'shadowWidth' => $_shadow_info[0],
@@ -845,17 +1046,18 @@ class GoogleMapAPI {
     /**
      * set the marker icon for ALL markers on the map
      */
-    function setMarkerIcon($iconImage,$iconShadowImage = '',$iconAnchorX = 'x',$iconAnchorY = 'x',$infoWindowAnchorX = 'x',$infoWindowAnchorY = 'x') {
-        $this->_icons = array($this->createMarkerIcon($iconImage,$iconShadowImage,$iconAnchorX,$iconAnchorY,$infoWindowAnchorX,$infoWindowAnchorY));
+    function setMarkerIcon($iconName,$iconImage,$iconShadowImage = '',$iconAnchorX = 'x',$iconAnchorY = 'x',$infoWindowAnchorX = 'x',$infoWindowAnchorY = 'x') {
+      $this->_icons[$iconName] = array($this->createMarkerIcon($iconName,$iconImage,$iconShadowImage,$iconAnchorX,$iconAnchorY,$infoWindowAnchorX,$infoWindowAnchorY));
     }
     
     /**
      * add an icon to go with the correspondingly added marker
      */
-    function addMarkerIcon($iconImage,$iconShadowImage = '',$iconAnchorX = 'x',$iconAnchorY = 'x',$infoWindowAnchorX = 'x',$infoWindowAnchorY = 'x') {
-        $this->_icons[] = $this->createMarkerIcon($iconImage,$iconShadowImage,$iconAnchorX,$iconAnchorY,$infoWindowAnchorX,$infoWindowAnchorY);
-        return count($this->_icons) - 1;
+    function addMarkerIcon($iconName,$iconImage,$iconShadowImage = '',$iconAnchorX = 'x',$iconAnchorY = 'x',$infoWindowAnchorX = 'x',$infoWindowAnchorY = 'x') {
+      $this->_icons[$iconName] = $this->createMarkerIcon($iconName,$iconImage,$iconShadowImage,$iconAnchorX,$iconAnchorY,$infoWindowAnchorX,$infoWindowAnchorY);
+      return count($this->_icons) - 1;
     }
+*/
 
     /**
      * print map header javascript (goes between <head></head>)
@@ -916,8 +1118,28 @@ class GoogleMapAPI {
         $_output .= " * http://www.phpinsider.com/php/code/GoogleMapAPI/\n";
         $_output .= " *************************************************/\n";
 
-	$_output .= "{\n";
-
+        $_output .= 'var points = [];' . "\n";
+        $_output .= 'var markers = new Array();' . "\n";
+        $_output .= 'var marker_aliases = new Array();' . "\n";
+        if($this->directions) {        
+            $_output .= 'var to_htmls = [];' . "\n";
+            $_output .= 'var from_htmls = [];' . "\n";
+            $_output .= 'var append_htmls = [];' . "\n";
+        }        
+        if($this->sidebar) {        
+	    $tmp = '';
+	    foreach( $this->_sidebar_list as $name => $contents )
+	      {
+		$tmp .= $contents;
+	      }
+            $_output .= 'var sidebar_html = \''.$tmp.'\';' . "\n";
+            $_output .= 'var marker_html = [];' . "\n";
+	    foreach( $this->_marker_list as $name => $contents )
+	      {
+		$_output .= "marker_html['{$name}'] = '{$contents}';\n";
+	      }
+	    
+        }
 
         $_output .= $this->getCreateMarkerJS();
 
@@ -927,15 +1149,16 @@ class GoogleMapAPI {
         $_output .= 'function isFunction(a) {return typeof a == \'function\';}' . "\n";
 
         if($this->sidebar) {        
-            $_output .= 'function click_sidebar(idx) {' . "\n";
-            $_output .= '  if(isArray(marker_html[idx])) { markers[idx].openInfoWindowTabsHtml(marker_html[idx]); }' . "\n";
-            $_output .= '  else { markers[idx].openInfoWindowHtml(marker_html[idx]); }' . "\n";
+            $_output .= 'function click_sidebar(alias) {' . "\n";
+            $_output .= '  name = marker_aliases[alias];'."\n";
+            $_output .= '  if(isArray(marker_html[name])) { markers[name].openInfoWindowTabsHtml(marker_html[alias]); }' . "\n";
+            $_output .= '  else { markers[name].openInfoWindowHtml(marker_html[alias]); }' . "\n";
             $_output .= '}' . "\n";
         }
-        $_output .= 'function showInfoWindow(idx,html) {' . "\n";
-        $_output .= $this->map_var_name.'.centerAtLatLng(points[idx]);' . "\n";
-        $_output .= 'markers[idx].openInfoWindowHtml(html);' . "\n";
-        $_output .= '}' . "\n";
+//         $_output .= 'function showInfoWindow(idx,html) {' . "\n";
+//         $_output .= $this->map_var_name.'.centerAtLatLng(points[idx]);' . "\n";
+//         $_output .= 'markers[idx].openInfoWindowHtml(html);' . "\n";
+//         $_output .= '}' . "\n";
         if($this->directions) {
             $_output .= 'function tohere(idx) {' . "\n";
             $_output .= 'markers[idx].openInfoWindowHtml(to_htmls[idx]);' . "\n";
@@ -945,45 +1168,24 @@ class GoogleMapAPI {
             $_output .= '}' . "\n";
         }
 
+//        $_output .= 'var counter = 0;' . "\n";
 
-        $_output .= 'var points = [];' . "\n";
-        $_output .= 'var markers = [];' . "\n";
-        $_output .= 'var counter = 0;' . "\n";
-        if($this->sidebar) {        
-            $_output .= 'var sidebar_html = "";' . "\n";
-            $_output .= 'var marker_html = [];' . "\n";
-        }
-
-        if($this->directions) {        
-            $_output .= 'var to_htmls = [];' . "\n";
-            $_output .= 'var from_htmls = [];' . "\n";
-        }        
-
-        if(!empty($this->_icons)) {
+        if(!empty($this->_icons)) 
+	  {
             $_output .= 'var icon = [];' . "\n";
-	    $exist_icn = array();
-            for($i = 0, $j = count($this->_icons); $i<$j; $i++) {
-                $info = $this->_icons[$i];
-
-                // hash the icon data to see if we've already got this one; if so, save some javascript
-                $icon_key = md5(serialize($info));
-		if( !isset($exist_icn[$icon_key]) )
-		{
-                    $_output .= "icon[$i] = new GIcon();\n";   
-                    $_output .= sprintf('icon[%s].image = "%s";',$i,$info['image']) . "\n";   
-                    if($info['shadow']) {
-                        $_output .= sprintf('icon[%s].shadow = "%s";',$i,$info['shadow']) . "\n";
-                        $_output .= sprintf('icon[%s].shadowSize = new GSize(%s,%s);',$i,$info['shadowWidth'],$info['shadowHeight']) . "\n";   
-                    }
-                    $_output .= sprintf('icon[%s].iconSize = new GSize(%s,%s);',$i,$info['iconWidth'],$info['iconHeight']) . "\n";   
-                    $_output .= sprintf('icon[%s].iconAnchor = new GPoint(%s,%s);',$i,$info['iconAnchorX'],$info['iconAnchorY']) . "\n";   
-                    $_output .= sprintf('icon[%s].infoWindowAnchor = new GPoint(%s,%s);',$i,$info['infoWindowAnchorX'],$info['infoWindowAnchorY']) . "\n";
-		    $exist_icn[$icon_key] = $i;
-                } else {
-                    $_output .= "icon[$i] = icon[$exist_icn[$icon_key]];\n";
-                }
-            }
-        }
+	    foreach( $this->_icons as $name => $info )
+	      {
+		$_output .= "icon['{$name}'] = new GIcon();\n";   
+		$_output .= sprintf('icon[\'%s\'].image = "%s";',$name,$info['image']) . "\n";   
+		if($info['shadow']) {
+		  $_output .= sprintf('icon[\'%s\'].shadow = "%s";',$name,$info['shadow']) . "\n";
+		  $_output .= sprintf('icon[\'%s\'].shadowSize = new GSize(%s,%s);',$name,$info['shadowWidth'],$info['shadowHeight']) . "\n";   
+		}
+		$_output .= sprintf('icon[\'%s\'].iconSize = new GSize(%s,%s);',$name,$info['iconWidth'],$info['iconHeight']) . "\n";   
+		$_output .= sprintf('icon[\'%s\'].iconAnchor = new GPoint(%s,%s);',$name,$info['iconAnchorX'],$info['iconAnchorY']) . "\n";   
+		$_output .= sprintf('icon[\'%s\'].infoWindowAnchor = new GPoint(%s,%s);',$name,$info['infoWindowAnchorX'],$info['infoWindowAnchorY']) . "\n";
+	      }
+	  }
                            
         $_output .= 'var '.$this->map_var_name.' = null;' . "\n";
                      
@@ -1005,7 +1207,9 @@ class GoogleMapAPI {
         }
         
         // zoom so that all markers are in the viewport
-        if($this->zoom_encompass && count($this->_markers) > 1) {
+        if($this->zoom_encompass && count($this->_markers) > 1) 
+	  {
+	    $this->calc_rectangle();
             // increase bounds by fudge factor to keep
             // markers away from the edges
             $_len_lon = $this->_max_lon - $this->_min_lon;
@@ -1042,10 +1246,10 @@ class GoogleMapAPI {
         $_output .= $this->getPolylineJS();
 
         if($this->sidebar) {
-            $_output .= sprintf('document.getElementById("%s").innerHTML = "<ul class=\"gmapSidebar\">"+ sidebar_html +"<\/ul>";', $this->sidebar_id) . "\n";
+            $_output .= sprintf('document.getElementById("%s").innerHTML = "<ul class=\"cggm_sidebar\">"+ sidebar_html +"<\/ul>";', $this->sidebar_id) . "\n";
         }
 
-        $_output .= '}' . "\n";        
+        //$_output .= '}' . "\n";        
        
         if(!empty($this->browser_alert)) {
             $_output .= '} else {' . "\n";
@@ -1067,42 +1271,72 @@ class GoogleMapAPI {
      * overridable function for generating js to add markers
      */
     function getAddMarkersJS() {
-        $SINGLE_TAB_WIDTH = 88;    // constant: width in pixels of each tab heading (set by google)
-        $i = 0;
+      //        $SINGLE_TAB_WIDTH = 88;    // constant: width in pixels of each tab heading (set by google)
+        $SINGLE_TAB_WIDTH = 150;    // constant: width in pixels of each tab heading (set by google)
         $_output = '';
+	$_output2 = '';
+	foreach( $this->_marker_aliases as $name => $alias )
+	  {
+	    $_output .= "marker_aliases['{$name}'] = '{$alias}';\n";
+	  }
+
         foreach($this->_markers as $_marker) {
+	  if( $this->directions )
+	    {
+	      $name = $_marker['name'];
+	      if( is_array($_marker['html']) && count($_marker['html']) > 0 )
+		{
+		  // note, this adds the direction stuff onto just the first tab.
+		  $_marker['html'][0] .= sprintf('<div class="cgmap_dir" style="white-space: nowrap;">%s<a href="javascript:tohere(\''.$name.'\')">%s</a> - <a href="javascript:fromhere(\''.$name.'\')">%s</a></div>',
+						 $this->driving_dir_text['dir_text'],
+						 $this->driving_dir_text['dir_tohere'],
+						 $this->driving_dir_text['dir_fromhere']);
+		}
+	      else
+		{
+		  $_marker['html'] .= sprintf('<div class="cgmap_dir" style="white-space: nowrap;">%s<a href="javascript:tohere(\''.$name.'\')">%s</a> - <a href="javascript:fromhere(\''.$name.'\')">%s</a></div>',
+					      $this->driving_dir_text['dir_text'],
+					      $this->driving_dir_text['dir_tohere'],
+					      $this->driving_dir_text['dir_fromhere']);
+		}
+	    }
+
             if(is_array($_marker['html'])) {
                 // warning: you can't have two tabs with the same header. but why would you want to?
                 $ti = 0;
                 $num_tabs = count($_marker['html']);
                 $tab_obs = array();
-                foreach($_marker['html'] as $tab => $info) {
-                    if($ti == 0 && $num_tabs > 2) {
-                        $width_style = sprintf(' style=\"width: %spx\"', $num_tabs * $SINGLE_TAB_WIDTH);
-                    } else {
-                        $width_style = '';
-                    }
+                foreach($_marker['html'] as $tab => $info) 
+		  {
+		    $width_style = '';
+		    //                     if($ti == 0 && $num_tabs > 2) {
+		    //                         $width_style = sprintf(' style=\"width: %spx\"', $num_tabs * $SINGLE_TAB_WIDTH);
+		    //                     }
                     $tab = str_replace('"','\"',$tab);
+		    
+		    
                     $info = str_replace('"','\"',$info);
 		    $info = str_replace(array("\n", "\r"), "", $info);
-                    $tab_obs[] = sprintf('new GInfoWindowTab("%s", "%s")', $tab, '<div id=\"gmapmarker\"'.$width_style.'>' . $info . '</div>');
-                    $ti++;
-                }
-                $iw_html = '[' . join(',',$tab_obs) . ']';
-            } else {
-                $iw_html = sprintf('"%s"',str_replace('"','\"','<div id="gmapmarker">' . str_replace(array("\n", "\r"), "", $_marker['html']) . '</div>'));
-            }
-            $_output .= sprintf('var point = new GLatLng(%s,%s);',$_marker['lat'],$_marker['lon']) . "\n";         
-            $_output .= sprintf('var marker = createMarker(point,"%s",%s, %s,"%s");',
-                                str_replace('"','\"',$_marker['title']),
-                                str_replace('/','\/',$iw_html),
-                                $i,
-                                str_replace('"','\"',$_marker['tooltip'])) . "\n";
-            //TODO: in above createMarker call, pass the index of the tab in which to put directions, if applicable
-            $_output .= $this->map_var_name.'.addOverlay(marker);' . "\n";
-            $i++;
-        }
-        return $_output;
+		    $tab_obs[] = sprintf('new GInfoWindowTab("%s", "%s")', $tab, '<div class=\"cggm_marker\"'.$width_style.'>' . $info . '</div>');
+		    $ti++;
+		  }
+		$iw_html = '[' . join(',',$tab_obs) . ']';
+	    } else {
+	      $iw_html = sprintf('"%s"',str_replace('"','\"','<div class="cggm_marker">' . str_replace(array("\n", "\r"), "", $_marker['html']) . '</div>'));
+	    }
+	    $_output .= sprintf('var point = new GLatLng(%s,%s);',$_marker['lat'],$_marker['lon']) . "\n";         
+	    $_output .= sprintf('var marker = createMarker(point,"%s","%s",%s, %s,"%s",%d);',
+				$_marker['name'],
+				str_replace('"','\"',$_marker['title']),
+				str_replace('/','\/',$iw_html),
+				"'".$_marker['icon']."'",
+				str_replace('"','\"',$_marker['tooltip']),
+				(is_array($_marker['html']))?count($_marker['html']):0) . "\n";
+	    $_output .= $_output2;
+	    $_output .= $this->map_var_name.'.addOverlay(marker);' . "\n";
+	    
+	}
+	return $_output;
     }
 
     /**
@@ -1122,67 +1356,45 @@ class GoogleMapAPI {
      * overridable function to generate the js for the js function for creating a marker.
      */
     function getCreateMarkerJS() {
-        $_output = 'function createMarker(point, title, html, n, tooltip) {' . "\n";
-        $_output .= 'if(n >= '. sizeof($this->_icons) .') { n = '. (sizeof($this->_icons) - 1) ."; }\n";
+        $_output = 'function createMarker(point, name, title, html, icon_name, tooltip, ntabs) {' . "\n";
         if(!empty($this->_icons)) {
-            $_output .= 'var marker = new GMarker(point,{\'icon\': icon[n], \'title\': tooltip});' . "\n";
+            $_output .= 'var marker = new GMarker(point,{\'icon\': icon[icon_name], \'title\': tooltip});' . "\n";
         } else {
             $_output .= 'var marker = new GMarker(point,{\'title\': tooltip});' . "\n";
         }
-        // TODO: make it so you can specify which tab you want the directions in.
-        if($this->directions) {
-            // WARNING: If you are using a tabbed info window AND directions: this uses an UNDOCUMENTED field
-            // of the GInfoWindowTab object, contentElem. Google may CHANGE this name or other aspects of their
-            // GInfoWindowTab implementation without warning and BREAK this code.
-            // NOTE: If you are NOT using a tabbed info window, you'll be fine.
-            $_output .= 'var tabFlag = isArray(html);' . "\n";
-            $_output .= 'if(!tabFlag) { html = [{"contentElem": html}]; }' . "\n";
-            $_output .= sprintf(
-                     "to_htmls[counter] = html[0].contentElem + '<p /><form class=\"gmapDir\" id=\"gmapDirTo\" style=\"white-space: nowrap;\" action=\"http://maps.google.com/maps\" method=\"get\" target=\"_blank\">' +
-                     '<span class=\"gmapDirHead\" id=\"gmapDirHeadTo\">%s<strong>%s</strong> - <a href=\"javascript:fromhere(' + counter + ')\">%s</a></span>' +
-                     '<p class=\"gmapDirItem\" id=\"gmapDirItemTo\"><label for=\"gmapDirSaddr\" class=\"gmapDirLabel\" id=\"gmapDirLabelTo\">%s<br /></label>' +
-                     '<input type=\"text\" size=\"40\" maxlength=\"40\" name=\"saddr\" class=\"gmapTextBox\" id=\"gmapDirSaddr\" value=\"\" onfocus=\"this.style.backgroundColor = \'#e0e0e0\';\" onblur=\"this.style.backgroundColor = \'#ffffff\';\" />' +
-                     '<span class=\"gmapDirBtns\" id=\"gmapDirBtnsTo\"><input value=\"%s\" type=\"%s\" class=\"gmapDirButton\" id=\"gmapDirButtonTo\" /></span></p>' +
-                     '<input type=\"hidden\" name=\"daddr\" value=\"' +
-                     point.y + ',' + point.x + \"(\" + title.replace(new RegExp(/\"/g),'&quot;') + \")\" + '\" /></form>';
-                      from_htmls[counter] = html[0].contentElem + '<p /><form class=\"gmapDir\" id=\"gmapDirFrom\" style=\"white-space: nowrap;\" action=\"http://maps.google.com/maps\" method=\"get\" target=\"_blank\">' +
-                     '<span class=\"gmapDirHead\" id=\"gmapDirHeadFrom\">%s<a href=\"javascript:tohere(' + counter + ')\">%s</a> - <strong>%s</strong></span>' +
-                     '<p class=\"gmapDirItem\" id=\"gmapDirItemFrom\"><label for=\"gmapDirSaddr\" class=\"gmapDirLabel\" id=\"gmapDirLabelFrom\">%s<br /></label>' +
-                     '<input type=\"text\" size=\"40\" maxlength=\"40\" name=\"saddr\" class=\"gmapTextBox\" id=\"gmapDirSaddr\" value=\"\" onfocus=\"this.style.backgroundColor = \'#e0e0e0\';\" onblur=\"this.style.backgroundColor = \'#ffffff\';\" />' +
-                     '<span class=\"gmapDirBtns\" id=\"gmapDirBtnsFrom\"><input value=\"%s\" type=\"%s\" class=\"gmapDirButton\" id=\"gmapDirButtonFrom\" /></span></p' +
-                     '<input type=\"hidden\" name=\"daddr\" value=\"' +
-                     point.y + ',' + point.x + \"(\" + title.replace(new RegExp(/\"/g),'&quot;') + \")\" + '\" /></form>';
-                     html[0].contentElem = html[0].contentElem + '<p /><div id=\"gmapDirHead\" class=\"gmapDir\" style=\"white-space: nowrap;\">%s<a href=\"javascript:tohere(' + counter + ')\">%s</a> - <a href=\"javascript:fromhere(' + counter + ')\">%s</a></div>';\n",
-                     $this->driving_dir_text['dir_text'],
-                     $this->driving_dir_text['dir_tohere'],
-                     $this->driving_dir_text['dir_fromhere'],
-                     $this->driving_dir_text['dir_to'],
-                     $this->driving_dir_text['to_button_value'],
-                     $this->driving_dir_text['to_button_type'],
-                     $this->driving_dir_text['dir_text'],
-                     $this->driving_dir_text['dir_tohere'],
-                     $this->driving_dir_text['dir_fromhere'],
-                     $this->driving_dir_text['dir_from'],
-                     $this->driving_dir_text['from_button_value'],
-                     $this->driving_dir_text['from_button_type'],
-                     $this->driving_dir_text['dir_text'],
-                     $this->driving_dir_text['dir_tohere'],
-                     $this->driving_dir_text['dir_fromhere']
-                    );
-            $_output .= 'if(!tabFlag) { html = html[0].contentElem; }';
-        }
+	if($this->directions) 
+	  {
+	    $_output .= 'title = title.replace(new RegExp(/"/g),\'&quot;\');'."\n";
+            $_output .= 'var str = point.y + \',\' + point.x + \'(\' + title + \')\''."\n";
+	    $_output .= sprintf(
+				'to_htmls[name] = \'<form class="cggm_dir" id="gmapDirTo" style="white-space: nowrap;" action="http://maps.google.com/maps" method="get" target="_blank"><span class="cggm_DirHead" id="cggm_DirHeadTo">%s<strong>%s</strong> - <a href="javascript:fromhere(\\\'\' + name + \'\\\')">%s</a></span><p class="cggm_DirItem" id="cggm_DirItemTo"><label for="cggm_DirSaddr" class="cggm_DirLabel" id="cggm_DirLabelTo">%s<br /></label><input type="text" size="40" maxlength="40" name="saddr" class="cggm_TextBox" id="cggm_DirSaddr" value="" onfocus="this.style.backgroundColor = \\\'#e0e0e0\\\';" onblur="this.style.backgroundColor = \\\'#ffffff\\\';" /><span class="cggm_DirBtns" id="cggm_DirBtnsTo"><input value="%s" type="%s" class="cggm_DirButton" id="cggm_DirButtonTo" /></span></p><input type="hidden" name="daddr" value="\'+str+\'" /></form>\';'."\n".
+				 'from_htmls[name] = \'<form class="cggm_Dir" id="cggm_DirFrom" style="white-space: nowrap;" action="http://maps.google.com/maps" method="get" target="_blank"><span class="cggm_DirHead" id="cggm_DirHeadFrom">%s<a href="javascript:tohere(\\\'\' + name + \'\\\')">%s</a> - <strong>%s</strong></span><p class="cggm_DirItem" id="cggm_DirItemFrom"><label for="cggm_DirSaddr" class="cggm_DirLabel" id="cggm_DirLabelFrom">%s<br /></label><input type="text" size="40" maxlength="40" name="saddr" class="cggm_TextBox" id="cggm_DirSaddr" value="" onfocus="this.style.backgroundColor = \\\'#e0e0e0\\\';" onblur="this.style.backgroundColor = \\\'#ffffff\\\';" /><span class="cggm_DirBtns" id="cggm_DirBtnsFrom"><input value="%s" type="%s" class="cggm_DirButton" id="cggm_DirButtonFrom" /></span></p><input type="hidden" name="daddr" value="\'+str+\'" /></form>\';'."\n",
+				 $this->driving_dir_text['dir_text'],
+				 $this->driving_dir_text['dir_tohere'],
+				 $this->driving_dir_text['dir_fromhere'],
+				 $this->driving_dir_text['dir_to'],
+				 $this->driving_dir_text['to_button_value'],
+				 $this->driving_dir_text['to_button_type'],
+				 $this->driving_dir_text['dir_text'],
+				 $this->driving_dir_text['dir_tohere'],
+				 $this->driving_dir_text['dir_fromhere'],
+				 $this->driving_dir_text['dir_from'],
+				 $this->driving_dir_text['from_button_value'],
+				 $this->driving_dir_text['from_button_type']);
+	  }
+
         
         if($this->info_window) {
-            $_output .= sprintf('if(isArray(html)) { GEvent.addListener(marker, "%s", function() { marker.openInfoWindowTabsHtml(html); }); }',$this->window_trigger) . "\n";
+            $_output .= sprintf('if(ntabs) { GEvent.addListener(marker, "%s", function() { marker.openInfoWindowTabsHtml(html); }); }',$this->window_trigger) . "\n";
             $_output .= sprintf('else { GEvent.addListener(marker, "%s", function() { marker.openInfoWindowHtml(html); }); }',$this->window_trigger) . "\n";
         }
-        $_output .= 'points[counter] = point;' . "\n";
-        $_output .= 'markers[counter] = marker;' . "\n";
-        if($this->sidebar) {        
-            $_output .= 'marker_html[counter] = html;' . "\n";
-            $_output .= "sidebar_html += '<li class=\"gmapSidebarItem\" id=\"gmapSidebarItem_'+ counter +'\"><a href=\"javascript:click_sidebar(' + counter + ')\">' + title + '<\/a><\/li>';" . "\n";
-        }
-        $_output .= 'counter++;' . "\n";
+        $_output .= 'points[name] = point;' . "\n";
+        $_output .= 'markers[name] = marker;' . "\n";
+	//$_output .= 'alert(name + \'-\' + markers.length);';
+//         if($this->sidebar) {        
+//             $_output .= 'marker_html[name] = html;' . "\n";
+//         }
+        //$_output .= 'counter++;' . "\n";
         $_output .= 'return marker;' . "\n";
         $_output .= '}' . "\n";
         return $_output;
